/* Private interface for struct ceelog_source

Copyright (C) 2012 Red Hat, Inc. All rights reserved.
This copyrighted material is made available to anyone wishing to use, modify,
copy, or redistribute it subject to the terms and conditions of the GNU General
Public License v.2.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301, USA.

Author: Miloslav Trmač <mitr@redhat.com> */

#ifndef SOURCE_PRIVATE_H
#define SOURCE_PRIVATE_H

#include <config.h>

#include <stdio.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <glib.h>

struct ceelog_event;

/* Source operations */
struct ceelog_source_ops
{
  /* Close SOURCE and free internal data. */
  void (*close) (struct ceelog_source *source);
  /* Read the next event from SOURCE.  If no event is currently available, but
     it may arrive later, block until it is available.

     Return the event (for ceelog_event_free ()) if it is available, or NULL if
     no more events will arrive (without setting ERROR), or NULL on error. */
  struct ceelog_event *(*read_event) (struct ceelog_source *source,
				      GError **error);
  /* Read the next event from SOURCE.  If no event is currently available,
     fail with CEELOG_ERROR_WOULD_BLOCK.

     Return the event (for ceelog_event_free ()) if it is available, or NULL if
     no more events will arrive (without setting ERROR), or NULL on error. */
  struct ceelog_event *(*read_event_nonblocking) (struct ceelog_source *source,
						  GError **error);
};

struct ceelog_source
{
  const struct ceelog_source_ops *ops;
  union
  {
    struct			/* For static_file source */
    {
      char *path; 		/* For g_free () */
      FILE *file;		/* For fclose () */
      char *line;		/* For free (), managed by getline (3) */
      size_t line_size;
    } file;
    struct
    {
      char *path;		/* For g_free () */
      FILE *file;		/* NULL or for fclose () */
      char *prevline;		/* For g_free () */

      int inotify_desc;
      int watch_desc;
    } logfile;
  } v;
};

/* Allocate and return a new source with OPS.
   The caller is expected to set up the source-specific data, if any. */
G_GNUC_INTERNAL
extern struct ceelog_source *ceelog_source_alloc
	(const struct ceelog_source_ops *ops);

#endif
